/* This file is part Tempura: Interval Temporal Logic interpreter.
 *  
 * Copyright (C) 1987-2016 Roger Hale, Ben Moszkowski, Antonio Cau 
 *
 * Tempura is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Tempura is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Tempura.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Changes made by  AC: Antonio Cau
 *                 BCM: Ben Moszkowski
 *
 * $Id: r-assign.c 267 2016-02-29 14:11:55Z cau $
 */


#include "system.h"



NODEPTR assign_curr (e1, e2)
NODEPTR e1, e2;
{
    NODEPTR val;
    NODEPTR loc;
    

    val = reduce_val (e2);
    if (!Ready) {
        val = reduce_val (e1);
        e1 = e2;
    }
    if (Ready) {
        loc = reduce_ref (e1);
            /*if (DebugLevel>2)
              printf("Assign_curr: history=%d\n",node_mem_history(loc));*/
        if (Ready) {
            if (!AssignAheadFlag && is_next (loc)) {
                return (assign (e1, val));
            }
            else {
                assign_val1 (loc, val);
                return (True);
            }
        }
    }
       
    return (True);
}


NODEPTR assign_next (e1, e2)
NODEPTR e1, e2;
{
    NODEPTR val;
    NODEPTR loc;
    
    val = reduce_val (e2);
    if (Ready) {
	if (!AssignAheadFlag)
	    return (assign (e1, val));
	NextFlag = T;
        loc = reduce_ref (e1);
            /*if (DebugLevel>2)
              printf("Assign_next: history=%d\n",node_mem_history(loc));*/
	NextFlag = F;
	if (Ready) {
	    assign_val1 (loc, val);
	    return (True);
	}
    }
    return (True);
}

/************************\
*                        *
* assignment statements  *
*                        *
\************************/

/* v = e (assignment) */
void reduce_EQ_stmt(e)
NODEPTR e;
{
    e = assign_curr (node_lnk_1(e), node_lnk_2(e));
    if (Ready)
        result (e);
}

/* e1 gets e2 */
void reduce_GETS(e)
NODEPTR e;
{
    keep (assign_next (node_lnk_1(e), node_lnk_2(e)));
}

/* stable e */
void reduce_STB(e)
NODEPTR e;
{
    keep (assign_next (node_lnk_1(e), node_lnk_1(e)));
}

/* e1 is e2 */
void reduce_IS(e)
NODEPTR e;
{
    always (assign_curr (node_lnk_1(e), node_lnk_2(e)));
}

/* e1 defaults e2 */
void reduce_DEFAULTS(e)
NODEPTR e;
{
    NODEPTR val = reduce_val (node_lnk_2(e));
    NODEPTR loc = reduce_ref (node_lnk_1(e));
    
    if (Ready) {
        assert0(node_mem_hasdefault(loc)==0,"only one default is allowed for a variable");
        node_mem_hasdefault(next_loc (loc)) = node_mem_hasdefault(curr_loc (loc)) =1;
        Retry = T;
        result (node2(IS,node_lnk_1(e),
                     node3(IF,node1(iUNDEFINED,node_lnk_1(e)),
                           val,node_lnk_1(e))));
    }
}

/* mem e1 */
void reduce_MEM(e)
NODEPTR e;
{
        /*Retry = T;
    
             result(node1(ALWAYS, node3(IF, node1(iUNDEFINED,node_lnk_1(e)),
                    node3(IF, node1(NOT,node1(iUNDEFINED,node1(PREV,node_lnk_1(e)))),
                          node2(EQ, node_lnk_1(e), node1(PREV,node_lnk_1(e))),True), True)));
    
        */
    
    warn0("The mem construct has been deprecated, use the existsf construct instead");
        /* result (node2(IS,node_lnk_1(e),
                 node3(IF,node1(iUNDEFINED,node_lnk_1(e)),
                       node3(IF,node1(iUNDEFINED,node1(PREV,node_lnk_1(e))),
                             node_lnk_1(e),
                             node1(PREV,node_lnk_1(e))),   
                             node_lnk_1(e))));*/
}


/* e1 <- e2 */
void reduce_ASSN(e)
NODEPTR e;
{
    NODEPTR val = reduce_val (node_lnk_2(e));
    if (Ready) {
            Retry = T;
            result (node1 (FIN, assign (node_lnk_1(e), val)));
    }
}

/* e1 := e2 */
void reduce_UASSN(e)
NODEPTR e;
{
    put_empty (False);
    e = assign_next (node_lnk_1(e), node_lnk_2(e));
    if (Ready)
        result (e);
}

/* e1 :- e2 */
void reduce_SKIPUASSN(e)
NODEPTR e;
{
    Retry = T;
    result(node2(AND,Skip,node2(UASSN,node_lnk_1(e),node_lnk_2(e))));
}
