/* This file is part Tempura: Interval Temporal Logic interpreter.
 *  
 * Copyright (C) 1987-2016 Roger Hale, Ben Moszkowski, Antonio Cau 
 *
 * Tempura is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Tempura is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Tempura.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Changes made by  AC: Antonio Cau
 *                 BCM: Ben Moszkowski
 *
 * $Id: r-symbol.c 267 2016-02-29 14:11:55Z cau $
 */


#include "system.h"




void noop(e)
NODEPTR e;
{
    error_break1 ("I'm sorry, Dave, I can't do \"%s\"", print_name (e));
}

void assert_stmt(e)
NODEPTR e;
{
    error_break1 ("\"%s\" can only be executed as a statement",print_name(e));
}

void assert_expr(e)
NODEPTR e;
{
    error_break1 ("\"%s\" cannot be executed as a statement", print_name(e));
    result (True);
}

/* 06/09/2001 added */
void assert_stmt_to_true(e)
NODEPTR e;
{
    NODEPTR e1;
    e1 = reduce_val (e);
    if (Ready) {
	if (e1 != True)
	  error_break0 ("Statement has reduced to false");
	e = True;
    };
    result(e);
}

void assert_true(e)
NODEPTR e;
{
    NODEPTR e1;
    e = reduce_val (e);
    if (!Ready) {
        switch (node_type(e)) {
            case NOT:
	        switch (node_type(e1 = node_lnk_1(e)) ) {
		    case OR:
	                Retry = T;
		        e = node2 (AND,
		                   node1 (NOT, node_lnk_1(e1)),
			           node1 (NOT, node_lnk_2(e1)));
                        break;
		    case AND:
	                Retry = T;
		        e = node2 (OR,
		                   node1 (NOT, node_lnk_1(e1)),
			           node1 (NOT, node_lnk_2(e1)));
                        break;
		    case NOT:
	                Retry = T;
		        e = node_lnk_1(e1);
		        break;
                    default:
		        e1 = assign_curr (node_lnk_1(e), False);
			if (Ready)
			    e = e1;
	                break;
                }
	        break;
	    default:
	        e1 = assign_curr (e, True);
		if (Ready)
		    e = e1;
	        break;
        }
    }
    result (e);
}


/*******************\
*                   *
*  tempura symbols  *
*                   *
\*******************/

void reduce_const(e)
NODEPTR e;
{
    Ready = val_defined (e);
}

void reduce_id_val(e)
NODEPTR e;
{
    NODEPTR loc=adj_loc1(mem_loc(e));
    NODEPTR ploc,cloc;
    NODEPTR val;
    unsigned int i;
    
        /*if (DebugLevel>1) {
        printf("***reduce_id_val, NF=%1d, PF=%d, MD=%1d\n",NextFlag,PrevFlag,mem_defined1(loc));
        show_mem_all1(loc);
        }*/
    
    if (is_framed_mem(loc)) {
        if (framing_condition) {
            cloc=curr_loc(loc);
            ploc=prev_loc1(loc,node_mem_history(loc)+1);
            if (mem_defined1 (cloc)) {
                if (!curr_val_defined(get_mem (cloc))) {
                        /*if (DebugLevel>1)
                          printf("framing is possible FO=%d,PPF=%d,SPF=%d\n",ForceOutput,PassProgressFlag,StateProgressFlag);*/
                    if (mem_defined1(ploc)){
                        if (prev_val_defined(val = get_mem(ploc),node_mem_history(loc)+1)) {
                            /* need to be carefull when val is a list*/
                            if (is_list(val)) {
                                assign_list_update(cloc,copy_prev_val(val,node_mem_history(loc)+1));
                            }
                            else {
                                put_mem1(cloc,copy_prev_val(val,node_mem_history(loc)+1));
                            }
                        }
                    }
                }
            }
            else {
                    /*if (DebugLevel>1)
                      printf("framing is possible FO=%d,PPF=%d,SPF=%d\n",ForceOutput,PassProgressFlag,StateProgressFlag);*/
                if (mem_defined1(ploc)) {
                    if (prev_val_defined(val = get_mem(ploc),node_mem_history(loc)+1)) {
                        /* need to be carefull when val is a list*/
                        if (is_list(val)) {
                            assign_list_update(cloc,copy_prev_val(val,node_mem_history(loc)+1));
                        }
                        else {
                            put_mem1(cloc,copy_prev_val(val,node_mem_history(loc)+1));
                        }
                    }
                }
            }
        }
    }
    
    /* we should get the right value!!*/
    if (is_curr (loc)) {
        if ((Ready = (mem_defined1 (loc) &&
                      curr_val_defined (val = get_mem (loc))))) {
            result (copy_curr_val(val));
        }
    }
    if (is_next(loc)) {
        if ((Ready = (mem_defined1 (loc) &&
                      next_val_defined (val = get_mem (loc))))) {
            result (copy_next_val(val));
        }
    }
    for (i=2; i<=1+node_mem_history(loc); i = i + 1) {
        if (is_prev1(loc,i)) {
            if ((Ready = (mem_defined1 (loc) &&
                          prev_val_defined (val = get_mem (loc),i)))) {
                result (copy_prev_val(val,i));
            }
        }
    }
}




void reduce_id_ref(e)
NODEPTR e;
{   NODEPTR loc=adj_loc1 (mem_loc (e)) ;
    NODEPTR ploc, cloc ;
    NODEPTR val;
    Ready = T;
        /*if (DebugLevel>1) {
        printf("***reduce_id_ref, NF=%1d, PF=%d, adj_loc=%p\n",NextFlag,PrevFlag,adj_loc1 (mem_loc (e)));
        show_mem_all1(loc);
        }*/
    if (is_framed_mem(loc)) {
        if (framing_condition) {
            cloc=curr_loc(loc);
            ploc=prev_loc1(loc,node_mem_history(loc)+1);
            if (mem_defined1 (cloc)) {
                if (!curr_val_defined(get_mem (cloc))) {
                        /*if (DebugLevel>1)
                          printf("framing is possible FO=%d,PPF=%d,SPF=%d\n",ForceOutput,PassProgressFlag,StateProgressFlag);*/
                    if (mem_defined1(ploc)) {
                        if (prev_val_defined(val=get_mem(ploc),node_mem_history(loc)+1)) {
                                /* need to be carefull when val is a list*/
                            if (is_list(val)) {
                                assign_list_update(cloc,copy_prev_val(val,node_mem_history(loc)+1));
                            }
                            else {
                                put_mem1(cloc,copy_prev_val(val,node_mem_history(loc)+1));
                            }
                        }
                    }
                }
            }
            else {
                    /*if (DebugLevel>1)
                      printf("framing is possible FO=%d,PPF=%d,SPF=%d\n",ForceOutput,PassProgressFlag,StateProgressFlag);*/
                if (mem_defined1(ploc)) {
                    if (prev_val_defined(val=get_mem(ploc),node_mem_history(loc)+1)) {
                        /* need to be carefull when val is a list*/
                        if (is_list(val)) {
                            assign_list_update(cloc,copy_prev_val(val,node_mem_history(loc)+1));
                        }
                        else {
                            put_mem1(cloc,copy_prev_val(val,node_mem_history(loc)+1));
                        }
                    }
                }
            }
        }    
    }
    ref_result (loc);
}


void reduce_EMPTY_stmt(e)
NODEPTR e;
{
    put_empty (True);
    Ready = T;
    result (True);
}

void reduce_EMPTY_val(e)
NODEPTR e;
{
    NODEPTR empty_loc = adj_loc1 (EmptyLoc);
    if ((Ready = mem_defined1 (empty_loc)))
        result (get_mem (empty_loc));
}

void reduce_EMPTY_ref(e)
NODEPTR e;
{
    Ready = T;
    ref_result (adj_loc1 (EmptyLoc));
}

void reduce_MORE_stmt(e)
NODEPTR e;
{
    put_empty (False);
    Ready = T;
    result (True);
}

void reduce_MORE_val(e)
NODEPTR e;
{
    NODEPTR e1;

    NODEPTR empty_loc = adj_loc1 (EmptyLoc);
    if ((Ready = mem_defined1 (empty_loc))) {
      e1 = get_mem (empty_loc);
      result (node_bval(e1) ? False : True);
    }
}

/* unit interval */
void reduce_SKIP(e)
NODEPTR e;
{
    put_empty (False);
    Ready = T;
    result (Empty);
}
